@@ -16,9 +16,14 @@
 
 use super::*;
 use crate::{
-    iana, util::expect_err, ContentType, CoseKeyBuilder, CoseRecipientBuilder,
+    cbor::Value, iana, util::expect_err, ContentType, CoseKeyBuilder, CoseRecipientBuilder,
     CoseSignatureBuilder, HeaderBuilder, TaggedCborSerializable,
 };
+use alloc::{
+    string::{String, ToString},
+    vec,
+    vec::Vec,
+};
 
 #[test]
 fn test_cose_recipient_decode() {
@@ -61,7 +66,7 @@ fn test_cose_recipient_decode() {
     ];
 
     for (i, (recipient, recipient_data)) in tests.iter().enumerate() {
-        let got = recipient.to_vec().unwrap();
+        let got = recipient.clone().to_vec().unwrap();
         assert_eq!(*recipient_data, hex::encode(&got), "case {}", i);
 
         let got = CoseRecipient::from_slice(&got).unwrap();
@@ -74,11 +79,11 @@ fn test_cose_recipient_decode_fail() {
     let tests = vec![
         (
             concat!(
-                "a2", // 2-map (should be tuple)
-                "40", // 0-bstr (special case for empty protected headers, rather than 41a0)
-                "a0", // 0-map
-                "40", // 0-bstr
-                "40", // 0-bstr
+                "a2",   // 2-map (should be tuple)
+                "40",   // 0-bstr (special case for empty protected headers, rather than 41a0)
+                "a0",   // 0-map
+                "4161", // 1-bstr
+                "40",   // 0-bstr
             ),
             "expected array",
         ),
@@ -164,7 +169,7 @@ fn test_cose_encrypt_decode() {
     ];
 
     for (i, (encrypt, encrypt_data)) in tests.iter().enumerate() {
-        let got = encrypt.to_vec().unwrap();
+        let got = encrypt.clone().to_vec().unwrap();
         assert_eq!(*encrypt_data, hex::encode(&got), "case {}", i);
 
         let got = CoseEncrypt::from_slice(&got).unwrap();
@@ -177,11 +182,11 @@ fn test_cose_encrypt_decode_fail() {
     let tests = vec![
         (
             concat!(
-                "a2", // 2-map (should be tuple)
-                "40", // 0-bstr (special case for empty protected headers, rather than 41a0)
-                "a0", // 0-map
-                "40", // 0-bstr
-                "40", // 0-bstr
+                "a2",   // 2-map (should be tuple)
+                "40",   // 0-bstr (special case for empty protected headers, rather than 41a0)
+                "a0",   // 0-map
+                "4161", // 1-bstr
+                "40",   // 0-bstr
             ),
             "expected array",
         ),
@@ -272,11 +277,11 @@ fn test_rfc8152_cose_encrypt_decode() {
                         )
                         .unprotected(
                             HeaderBuilder::new()
-                                .value(iana::HeaderAlgorithmParameter::EphemeralKey as i128,
+                                .value(iana::HeaderAlgorithmParameter::EphemeralKey as i64,
                                        CoseKeyBuilder::new_ec2_pub_key_y_sign(iana::EllipticCurve::P_256,
                                                                               hex::decode("98f50a4ff6c05861c8860d13a638ea56c3f5ad7590bbfbf054e1c7b4d91d6280").unwrap(),
                                                                               true)
-                                       .build().to_cbor_value())
+                                       .build().to_cbor_value().unwrap())
                                 .key_id(b"meriadoc.brandybuck@buckland.example".to_vec())
                                 .build(),
                         )
@@ -315,8 +320,8 @@ fn test_rfc8152_cose_encrypt_decode() {
                                .unprotected(
                                    HeaderBuilder::new()
                                        .key_id(b"our-secret".to_vec())
-                                       .value(iana::HeaderAlgorithmParameter::Salt as i128,
-                                              cbor::Value::Bytes(b"aabbccddeeffgghh".to_vec()))
+                                       .value(iana::HeaderAlgorithmParameter::Salt as i64,
+                                              Value::ByteString(b"aabbccddeeffgghh".to_vec()))
                                        .build())
                                .ciphertext(vec![])
                                .build())
@@ -361,11 +366,11 @@ fn test_rfc8152_cose_encrypt_decode() {
                                        .build())
                                .unprotected(
                                    HeaderBuilder::new()
-                                       .value(iana::HeaderAlgorithmParameter::EphemeralKey as i128,
+                                       .value(iana::HeaderAlgorithmParameter::EphemeralKey as i64,
                                               CoseKeyBuilder::new_ec2_pub_key_y_sign(iana::EllipticCurve::P_256,
                                                                                      hex::decode("98f50a4ff6c05861c8860d13a638ea56c3f5ad7590bbfbf054e1c7b4d91d6280").unwrap(),
                                                                                      true)
-                                              .build().to_cbor_value())
+                                              .build().to_cbor_value().unwrap())
                                        .key_id(b"meriadoc.brandybuck@buckland.example".to_vec())
                                        .build())
                         .ciphertext(vec![])
@@ -420,12 +425,12 @@ fn test_rfc8152_cose_encrypt_decode() {
                                .unprotected(HeaderBuilder::new()
                                             .key_id(b"meriadoc.brandybuck@buckland.example".to_vec())
                                             .value(
-                                                iana::HeaderAlgorithmParameter::StaticKeyId as i128,
-                                                cbor::Value::Bytes(b"peregrin.took@tuckborough.example".to_vec())
+                                                iana::HeaderAlgorithmParameter::StaticKeyId as i64,
+                                                Value::ByteString(b"peregrin.took@tuckborough.example".to_vec())
                                             )
                                             .value(
-                                                iana::HeaderAlgorithmParameter::PartyUNonce as i128,
-                                                cbor::Value::Bytes(hex::decode("0101").unwrap())
+                                                iana::HeaderAlgorithmParameter::PartyUNonce as i64,
+                                                Value::ByteString(hex::decode("0101").unwrap())
                                             )
                                             .build())
                                .ciphertext(hex::decode("41e0d76f579dbd0d936a662d54d8582037de2e366fde1c62").unwrap())
@@ -458,7 +463,7 @@ fn test_rfc8152_cose_encrypt_decode() {
     ];
 
     for (i, (encrypt, encrypt_data)) in tests.iter().enumerate() {
-        let got = encrypt.to_tagged_vec().unwrap();
+        let got = encrypt.clone().to_tagged_vec().unwrap();
         assert_eq!(*encrypt_data, hex::encode(&got), "case {}", i);
 
         let got = CoseEncrypt::from_tagged_slice(&got).unwrap();
@@ -490,7 +495,7 @@ fn test_cose_encrypt0_decode() {
     ];
 
     for (i, (encrypt, encrypt_data)) in tests.iter().enumerate() {
-        let got = encrypt.to_vec().unwrap();
+        let got = encrypt.clone().to_vec().unwrap();
         assert_eq!(*encrypt_data, hex::encode(&got), "case {}", i);
 
         let got = CoseEncrypt0::from_slice(&got).unwrap();
@@ -503,11 +508,11 @@ fn test_cose_encrypt0_decode_fail() {
     let tests = vec![
         (
             concat!(
-                "a2", // 2-map (should be tuple)
-                "40", // 0-bstr (special case for empty protected headers, rather than 41a0)
-                "a0", // 0-map
-                "40", // 0-bstr
-                "40", // 0-bstr
+                "a2",   // 2-map (should be tuple)
+                "40",   // 0-bstr (special case for empty protected headers, rather than 41a0)
+                "a0",   // 0-map
+                "4100", // 1-bstr
+                "40",   // 0-bstr
             ),
             "expected array",
         ),
@@ -621,7 +626,7 @@ fn test_rfc8152_cose_encrypt0_decode() {
     ];
 
     for (i, (encrypt, encrypt_data)) in tests.iter().enumerate() {
-        let got = encrypt.to_tagged_vec().unwrap();
+        let got = encrypt.clone().to_tagged_vec().unwrap();
         assert_eq!(*encrypt_data, hex::encode(&got), "case {}", i);
 
         let got = CoseEncrypt0::from_tagged_slice(&got).unwrap();
@@ -665,49 +670,45 @@ fn test_cose_recipient_roundtrip() {
     let external_aad = b"This is the external aad";
     let cipher = FakeCipher {};
 
-    let protected = HeaderBuilder::new()
-        .algorithm(iana::Algorithm::ES256)
-        .key_id(b"11".to_vec())
-        .build();
-    let mut recipient = CoseRecipientBuilder::new()
-        .protected(protected)
-        .create_ciphertext(
-            EncryptionContext::EncRecipient,
-            pt,
-            external_aad,
-            |pt, aad| cipher.encrypt(pt, aad).unwrap(),
-        )
-        .build();
+    for context in [
+        EncryptionContext::EncRecipient,
+        EncryptionContext::MacRecipient,
+        EncryptionContext::RecRecipient,
+    ] {
+        let protected = HeaderBuilder::new()
+            .algorithm(iana::Algorithm::ES256)
+            .key_id(b"11".to_vec())
+            .build();
 
-    let recovered_pt = recipient
-        .decrypt(EncryptionContext::EncRecipient, external_aad, |ct, aad| {
-            cipher.decrypt(ct, aad)
-        })
-        .unwrap();
-    assert_eq!(&pt[..], recovered_pt);
+        let mut recipient = CoseRecipientBuilder::new()
+            .protected(protected)
+            .create_ciphertext(context, pt, external_aad, |pt, aad| {
+                cipher.encrypt(pt, aad).unwrap()
+            })
+            .build();
 
-    // Changing an unprotected header leaves the ciphertext decipherable.
-    recipient.unprotected.content_type = Some(ContentType::Text("text/plain".to_owned()));
-    assert!(recipient
-        .decrypt(EncryptionContext::EncRecipient, external_aad, |ct, aad| {
-            cipher.decrypt(ct, aad)
-        })
-        .is_ok());
+        let recovered_pt = recipient
+            .decrypt(context, external_aad, |ct, aad| cipher.decrypt(ct, aad))
+            .unwrap();
+        assert_eq!(&pt[..], recovered_pt);
 
-    // Providing a different `aad` means the signature won't validate.
-    assert!(recipient
-        .decrypt(EncryptionContext::EncRecipient, b"not aad", |ct, aad| {
-            cipher.decrypt(ct, aad)
-        })
-        .is_err());
+        // Changing an unprotected header leaves the ciphertext decipherable.
+        recipient.unprotected.content_type = Some(ContentType::Text("text/plain".to_owned()));
+        assert!(recipient
+            .decrypt(context, external_aad, |ct, aad| { cipher.decrypt(ct, aad) })
+            .is_ok());
 
-    // Changing a protected header invalidates the signature.
-    recipient.protected = Header::default();
-    assert!(recipient
-        .decrypt(EncryptionContext::EncRecipient, external_aad, |ct, aad| {
-            cipher.decrypt(ct, aad)
-        })
-        .is_err());
+        // Providing a different `aad` means the signature won't validate.
+        assert!(recipient
+            .decrypt(context, b"not aad", |ct, aad| { cipher.decrypt(ct, aad) })
+            .is_err());
+
+        // Changing a protected header invalidates the signature.
+        recipient.protected = Header::default();
+        assert!(recipient
+            .decrypt(context, external_aad, |ct, aad| { cipher.decrypt(ct, aad) })
+            .is_err());
+    }
 }
 
 #[test]
