@@ -15,14 +15,13 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 use super::*;
-use crate::{iana, util::expect_err, CborSerializable, Label};
-use maplit::btreemap;
-use serde_cbor as cbor;
-
-// The most negative integer value that can be encoded in CBOR is:
-//    0x3B (0b001_11011) 0xFF 0xFF 0xFF 0xFF 0xFF 0xFF 0xFF 0xFF
-// which is -18_446_744_073_709_551_616 (-1 - 18_446_744_073_709_551_615).
-const MOST_NEGATIVE_NINT: i128 = -18_446_744_073_709_551_616i128;
+use crate::{
+    cbor::values::{SimpleValue, Value},
+    iana,
+    util::expect_err,
+    CborSerializable, Label,
+};
+use alloc::{borrow::ToOwned, vec};
 
 #[test]
 fn test_header_encode() {
@@ -43,13 +42,13 @@ fn test_header_encode() {
         ),
         (
             Header {
-                alg: Some(Algorithm::PrivateUse(MOST_NEGATIVE_NINT)),
+                alg: Some(Algorithm::PrivateUse(i64::MIN)),
                 ..Default::default()
             },
             concat!(
                 "a1", // 1-map
                 "01",
-                "3bffffffffffffffff", // 1 (alg) => -lots
+                "3b7fffffffffffffff", // 1 (alg) => -lots
             ),
         ),
         (
@@ -59,10 +58,10 @@ fn test_header_encode() {
                 content_type: Some(ContentType::Assigned(iana::CoapContentFormat::CoseEncrypt0)),
                 key_id: vec![1, 2, 3],
                 iv: vec![1, 2, 3],
-                rest: btreemap! {
-                    Label::Int(0x46) => cbor::Value::Integer(0x47),
-                    Label::Int(0x66) => cbor::Value::Integer(0x67),
-                },
+                rest: vec![
+                    (Label::Int(0x46), Value::Unsigned(0x47)),
+                    (Label::Int(0x66), Value::Unsigned(0x67)),
+                ],
                 ..Default::default()
             },
             concat!(
@@ -83,10 +82,10 @@ fn test_header_encode() {
                 content_type: Some(ContentType::Text("a/b".to_owned())),
                 key_id: vec![1, 2, 3],
                 iv: vec![1, 2, 3],
-                rest: btreemap! {
-                    Label::Int(0x46) => cbor::Value::Integer(0x47),
-                    Label::Text("a".to_owned()) => cbor::Value::Integer(0x47),
-                },
+                rest: vec![
+                    (Label::Int(0x46), Value::Unsigned(0x47)),
+                    (Label::Text("a".to_owned()), Value::Unsigned(0x47)),
+                ],
                 counter_signatures: vec![CoseSignature {
                     signature: vec![1, 2, 3],
                     ..Default::default()
@@ -113,10 +112,10 @@ fn test_header_encode() {
                 content_type: Some(ContentType::Text("a/b".to_owned())),
                 key_id: vec![1, 2, 3],
                 iv: vec![1, 2, 3],
-                rest: btreemap! {
-                    Label::Int(0x46) => cbor::Value::Integer(0x47),
-                    Label::Text("a".to_owned()) => cbor::Value::Integer(0x47),
-                },
+                rest: vec![
+                    (Label::Int(0x46), Value::Unsigned(0x47)),
+                    (Label::Text("a".to_owned()), Value::Unsigned(0x47)),
+                ],
                 counter_signatures: vec![
                     CoseSignature {
                         signature: vec![1, 2, 3],
@@ -143,9 +142,19 @@ fn test_header_encode() {
                 "6161", "1847", // "a" => 47
             ),
         ),
+        (
+            HeaderBuilder::new()
+                .add_critical(iana::HeaderParameter::Alg)
+                .add_critical(iana::HeaderParameter::Alg)
+                .build(),
+            concat!(
+                "a1", // 1-map
+                "02", "820101", // crit => 2-arr [1, 1]
+            ),
+        ),
     ];
     for (i, (header, header_data)) in tests.iter().enumerate() {
-        let got = cbor::ser::to_vec(&header).unwrap();
+        let got = header.clone().to_vec().unwrap();
         assert_eq!(*header_data, hex::encode(&got), "case {}", i);
 
         let got = Header::from_slice(&got).unwrap();
@@ -176,7 +185,7 @@ fn test_header_decode_fail() {
                 "a1", // 1-map
                 "02", "4101", // 2 (crit) => bstr (invalid value type)
             ),
-            "expected array value",
+            "expected array",
         ),
         (
             concat!(
@@ -232,7 +241,7 @@ fn test_header_decode_fail() {
                 "a1", // 1-map
                 "03", "60", // 3 (content-type) => invalid value ""
             ),
-            "expected non-empty string",
+            "expected non-empty tstr",
         ),
         (
             concat!(
@@ -288,7 +297,7 @@ fn test_header_decode_fail() {
                 "a1", // 1-map
                 "07", "80", // 7 (counter-sig) => 0-arr
             ),
-            "expected non-empty array",
+            "expected non-empty sig array",
         ),
         (
             concat!(
@@ -316,9 +325,7 @@ fn test_header_decode_fail() {
     }
 }
 
-// TODO(#1): get serde_cbor to generate an error on duplicate keys in map
 #[test]
-#[ignore]
 fn test_header_decode_dup_fail() {
     let tests = vec![
         (
@@ -328,7 +335,7 @@ fn test_header_decode_dup_fail() {
                 "1866", "1867", // 66 => 67
                 "1866", "1847", // 66 => 47
             ),
-            "expected unique map label",
+            "OutOfOrderKey",
         ),
         (
             concat!(
@@ -337,7 +344,7 @@ fn test_header_decode_dup_fail() {
                 "1866", "1867", // 66 => 67
                 "01", "01", // 1 (alg) => A128GCM (duplicate label)
             ),
-            "expected unique map label",
+            "OutOfOrderKey",
         ),
     ];
     for (header_data, err_msg) in tests.iter() {
@@ -348,26 +355,29 @@ fn test_header_decode_dup_fail() {
 }
 
 #[test]
-fn test_header_encode_int_out_of_range_fail() {
-    // Unfortunately, it is possible to build Rust COSE structures that hold values that
-    // cannot be serialized to CBOR -- serde_cbor uses i128 for integer values for convenience,
-    // but the serializable range is more like an i65 (!).
-    let header = HeaderBuilder::new()
-        .value(
-            0x10000000000000000i128,
-            cbor::Value::Text("boom!".to_owned()),
-        )
-        .build();
-    let result = cbor::ser::to_vec(&header);
-    expect_err(result, "stored in CBOR");
-
-    let header = Header {
-        // "What we do, is if we need that extra push over the cliff, you know what we do?"
-        alg: Some(Algorithm::PrivateUse(MOST_NEGATIVE_NINT - 1)),
-        ..Default::default()
-    };
-    let result = cbor::ser::to_vec(&header);
-    expect_err(result, "stored in CBOR");
+fn test_header_encode_dup_fail() {
+    let tests = vec![
+        Header {
+            alg: Some(Algorithm::Assigned(iana::Algorithm::A128GCM)),
+            crit: vec![RegisteredLabel::Assigned(iana::HeaderParameter::Alg)],
+            content_type: Some(ContentType::Assigned(iana::CoapContentFormat::CoseEncrypt0)),
+            key_id: vec![1, 2, 3],
+            iv: vec![1, 2, 3],
+            rest: vec![
+                (Label::Int(0x46), Value::Unsigned(0x47)),
+                (Label::Int(0x46), Value::Unsigned(0x67)),
+            ],
+            ..Default::default()
+        },
+        HeaderBuilder::new()
+            .text_value("doop".to_owned(), Value::Unsigned(1))
+            .text_value("doop".to_owned(), Value::Unsigned(2))
+            .build(),
+    ];
+    for header in tests {
+        let result = header.clone().to_vec();
+        expect_err(result, "encode CBOR failure");
+    }
 }
 
 #[test]
@@ -388,8 +398,8 @@ fn test_header_builder() {
                 .key_id(vec![1, 2, 3])
                 .partial_iv(vec![4, 5, 6]) // removed by .iv() call
                 .iv(vec![1, 2, 3])
-                .value(0x46, cbor::Value::Integer(0x47))
-                .value(0x66, cbor::Value::Integer(0x67))
+                .value(0x46, Value::Unsigned(0x47))
+                .value(0x66, Value::Unsigned(0x67))
                 .build(),
             Header {
                 alg: Some(Algorithm::Assigned(iana::Algorithm::A128GCM)),
@@ -400,10 +410,10 @@ fn test_header_builder() {
                 content_type: Some(ContentType::Assigned(iana::CoapContentFormat::CoseEncrypt0)),
                 key_id: vec![1, 2, 3],
                 iv: vec![1, 2, 3],
-                rest: btreemap! {
-                    Label::Int(0x46) => cbor::Value::Integer(0x47),
-                    Label::Int(0x66) => cbor::Value::Integer(0x67),
-                },
+                rest: vec![
+                    (Label::Int(0x46), Value::Unsigned(0x47)),
+                    (Label::Int(0x66), Value::Unsigned(0x67)),
+                ],
                 ..Default::default()
             },
         ),
@@ -439,5 +449,7 @@ fn test_header_builder() {
 #[should_panic]
 fn test_header_builder_core_param_panic() {
     // Attempting to set a core header parameter (in range [1,7]) via `.param()` panics.
-    let _hdr = HeaderBuilder::new().value(1, cbor::Value::Null).build();
+    let _hdr = HeaderBuilder::new()
+        .value(1, Value::Simple(SimpleValue::NullValue))
+        .build();
 }
